@@ -261,11 +261,12 @@ def read_images_binary(path_to_model_file):
             qvec = np.array(binary_image_properties[1:5])
             tvec = np.array(binary_image_properties[5:8])
             camera_id = binary_image_properties[8]
-            image_name = ""
+            binary_image_name = b""
             current_char = read_next_bytes(fid, 1, "c")[0]
             while current_char != b"\x00":  # look for the ASCII 0 entry
-                image_name += current_char.decode("utf-8")
+                binary_image_name += current_char
                 current_char = read_next_bytes(fid, 1, "c")[0]
+            image_name = binary_image_name.decode("utf-8")
             num_points2D = read_next_bytes(
                 fid, num_bytes=8, format_char_sequence="Q"
             )[0]
@@ -577,7 +578,7 @@ def main():
     parser.add_argument(
         "--output_format",
         choices=[".bin", ".txt"],
-        help="outut model format",
+        help="output model format",
         default=".txt",
     )
     args = parser.parse_args()
