@@ -30,10 +30,12 @@
 #include "colmap/controllers/feature_extraction.h"
 
 #include "colmap/feature/sift.h"
+#include "colmap/geometry/gps.h"
 #include "colmap/scene/database.h"
 #include "colmap/util/cuda.h"
 #include "colmap/util/misc.h"
 #include "colmap/util/opengl_utils.h"
+#include "colmap/util/timer.h"
 
 #include <numeric>
 
@@ -86,6 +88,7 @@ struct ImageData {
 
   Camera camera;
   Image image;
+  PosePrior pose_prior;
   Bitmap bitmap;
   Bitmap mask;
 
@@ -152,7 +155,7 @@ class SiftFeatureExtractorThread : public Thread {
         camera_mask_(camera_mask),
         input_queue_(input_queue),
         output_queue_(output_queue) {
-    CHECK(sift_options_.Check());
+    THROW_CHECK(sift_options_.Check());
 
 #if !defined(COLMAP_CUDA_ENABLED)
     if (sift_options_.use_gpu) {
@@ -165,8 +168,8 @@ class SiftFeatureExtractorThread : public Thread {
   void Run() override {
     if (sift_options_.use_gpu) {
 #if !defined(COLMAP_CUDA_ENABLED)
-      CHECK(opengl_context_);
-      CHECK(opengl_context_->MakeCurrent());
+      THROW_CHECK_NOTNULL(opengl_context_);
+      THROW_CHECK(opengl_context_->MakeCurrent());
 #endif
     }
 
@@ -290,15 +293,6 @@ class FeatureWriterThread : public Thread {
             "  Focal Length:    %.2fpx%s",
             image_data.camera.MeanFocalLength(),
             image_data.camera.has_prior_focal_length ? " (Prior)" : "");
-        const Eigen::Vector3d& translation_prior =
-            image_data.image.CamFromWorldPrior().translation;
-        if (translation_prior.array().isFinite().any()) {
-          LOG(INFO) << StringPrintf(
-              "  GPS:             LAT=%.3f, LON=%.3f, ALT=%.3f",
-              translation_prior.x(),
-              translation_prior.y(),
-              translation_prior.z());
-        }
         LOG(INFO) << StringPrintf("  Features:        %d",
                                   image_data.keypoints.size());
 
@@ -306,6 +300,15 @@ class FeatureWriterThread : public Thread {
 
         if (image_data.image.ImageId() == kInvalidImageId) {
           image_data.image.SetImageId(database_->WriteImage(image_data.image));
+          if (image_data.pose_prior.IsValid()) {
+            LOG(INFO) << StringPrintf(
+                "  GPS:             LAT=%.3f, LON=%.3f, ALT=%.3f",
+                image_data.pose_prior.position.x(),
+                image_data.pose_prior.position.y(),
+                image_data.pose_prior.position.z());
+            database_->WritePosePrior(image_data.image.ImageId(),
+                                      image_data.pose_prior);
+          }
         }
 
         if (!database_->ExistsKeypoints(image_data.image.ImageId())) {
@@ -337,8 +340,8 @@ class FeatureExtractorController : public Thread {
         sift_options_(sift_options),
         database_(reader_options_.database_path),
         image_reader_(reader_options_, &database_) {
-    CHECK(reader_options_.Check());
-    CHECK(sift_options_.Check());
+    THROW_CHECK(reader_options_.Check());
+    THROW_CHECK(sift_options_.Check());
 
     std::shared_ptr<Bitmap> camera_mask;
     if (!reader_options_.camera_mask_path.empty()) {
@@ -353,7 +356,7 @@ class FeatureExtractorController : public Thread {
     }
 
     const int num_threads = GetEffectiveNumThreads(sift_options_.num_threads);
-    CHECK_GT(num_threads, 0);
+    THROW_CHECK_GT(num_threads, 0);
 
     // Make sure that we only have limited number of objects in the queue to
     // avoid excess in memory usage since images and features take lots of
@@ -375,12 +378,12 @@ class FeatureExtractorController : public Thread {
     if (!sift_options_.domain_size_pooling &&
         !sift_options_.estimate_affine_shape && sift_options_.use_gpu) {
       std::vector<int> gpu_indices = CSVToVector<int>(sift_options_.gpu_index);
-      CHECK_GT(gpu_indices.size(), 0);
+      THROW_CHECK_GT(gpu_indices.size(), 0);
 
 #if defined(COLMAP_CUDA_ENABLED)
       if (gpu_indices.size() == 1 && gpu_indices[0] == -1) {
         const int num_cuda_devices = GetNumCudaDevices();
-        CHECK_GT(num_cuda_devices, 0);
+        THROW_CHECK_GT(num_cuda_devices, 0);
         gpu_indices.resize(num_cuda_devices);
         std::iota(gpu_indices.begin(), gpu_indices.end(), 0);
       }
@@ -428,6 +431,8 @@ class FeatureExtractorController : public Thread {
  private:
   void Run() override {
     PrintHeading1("Feature extraction");
+    Timer run_timer;
+    run_timer.Start();
 
     for (auto& resizer : resizers_) {
       resizer->Start();
@@ -457,6 +462,7 @@ class FeatureExtractorController : public Thread {
       ImageData image_data;
       image_data.status = image_reader_.Next(&image_data.camera,
                                              &image_data.image,
+                                             &image_data.pose_prior,
                                              &image_data.bitmap,
                                              &image_data.mask);
 
@@ -465,9 +471,9 @@ class FeatureExtractorController : public Thread {
       }
 
       if (sift_options_.max_image_size > 0) {
-        CHECK(resizer_queue_->Push(std::move(image_data)));
+        THROW_CHECK(resizer_queue_->Push(std::move(image_data)));
       } else {
-        CHECK(extractor_queue_->Push(std::move(image_data)));
+        THROW_CHECK(extractor_queue_->Push(std::move(image_data)));
       }
     }
 
@@ -487,7 +493,7 @@ class FeatureExtractorController : public Thread {
     writer_queue_->Stop();
     writer_->Wait();
 
-    GetTimer().PrintMinutes();
+    run_timer.PrintMinutes();
   }
 
   const ImageReaderOptions reader_options_;
@@ -516,6 +522,8 @@ class FeatureImporterController : public Thread {
  private:
   void Run() override {
     PrintHeading1("Feature import");
+    Timer run_timer;
+    run_timer.Start();
 
     if (!ExistsDir(import_path_)) {
       LOG(ERROR) << "Import directory does not exist.";
@@ -537,8 +545,9 @@ class FeatureImporterController : public Thread {
       // Load image data and possibly save camera to database.
       Camera camera;
       Image image;
+      PosePrior pose_prior;
       Bitmap bitmap;
-      if (image_reader.Next(&camera, &image, &bitmap, nullptr) !=
+      if (image_reader.Next(&camera, &image, &pose_prior, &bitmap, nullptr) !=
           ImageReader::Status::SUCCESS) {
         continue;
       }
@@ -556,6 +565,9 @@ class FeatureImporterController : public Thread {
 
         if (image.ImageId() == kInvalidImageId) {
           image.SetImageId(database.WriteImage(image));
+          if (pose_prior.IsValid()) {
+            database.WritePosePrior(image.ImageId(), pose_prior);
+          }
         }
 
         if (!database.ExistsKeypoints(image.ImageId())) {
@@ -570,7 +582,7 @@ class FeatureImporterController : public Thread {
       }
     }
 
-    GetTimer().PrintMinutes();
+    run_timer.PrintMinutes();
   }
 
   const ImageReaderOptions reader_options_;
