@@ -40,134 +40,6 @@
 
 namespace colmap {
 
-FeatureMatcherCache::FeatureMatcherCache(const size_t cache_size,
-                                         const Database* database)
-    : cache_size_(cache_size), database_(database) {
-  CHECK_NOTNULL(database_);
-}
-
-void FeatureMatcherCache::Setup() {
-  std::vector<Camera> cameras = database_->ReadAllCameras();
-  cameras_cache_.reserve(cameras.size());
-  for (Camera& camera : cameras) {
-    cameras_cache_.emplace(camera.camera_id, std::move(camera));
-  }
-
-  std::vector<Image> images = database_->ReadAllImages();
-  images_cache_.reserve(images.size());
-  for (Image& image : images) {
-    images_cache_.emplace(image.ImageId(), std::move(image));
-  }
-
-  keypoints_cache_ =
-      std::make_unique<LRUCache<image_t, std::shared_ptr<FeatureKeypoints>>>(
-          cache_size_, [this](const image_t image_id) {
-            return std::make_shared<FeatureKeypoints>(
-                database_->ReadKeypoints(image_id));
-          });
-
-  descriptors_cache_ =
-      std::make_unique<LRUCache<image_t, std::shared_ptr<FeatureDescriptors>>>(
-          cache_size_, [this](const image_t image_id) {
-            return std::make_shared<FeatureDescriptors>(
-                database_->ReadDescriptors(image_id));
-          });
-
-  keypoints_exists_cache_ = std::make_unique<LRUCache<image_t, bool>>(
-      images_cache_.size(), [this](const image_t image_id) {
-        return database_->ExistsKeypoints(image_id);
-      });
-
-  descriptors_exists_cache_ = std::make_unique<LRUCache<image_t, bool>>(
-      images_cache_.size(), [this](const image_t image_id) {
-        return database_->ExistsDescriptors(image_id);
-      });
-}
-
-const Camera& FeatureMatcherCache::GetCamera(const camera_t camera_id) const {
-  return cameras_cache_.at(camera_id);
-}
-
-const Image& FeatureMatcherCache::GetImage(const image_t image_id) const {
-  return images_cache_.at(image_id);
-}
-
-std::shared_ptr<FeatureKeypoints> FeatureMatcherCache::GetKeypoints(
-    const image_t image_id) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return keypoints_cache_->Get(image_id);
-}
-
-std::shared_ptr<FeatureDescriptors> FeatureMatcherCache::GetDescriptors(
-    const image_t image_id) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return descriptors_cache_->Get(image_id);
-}
-
-FeatureMatches FeatureMatcherCache::GetMatches(const image_t image_id1,
-                                               const image_t image_id2) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return database_->ReadMatches(image_id1, image_id2);
-}
-
-std::vector<image_t> FeatureMatcherCache::GetImageIds() const {
-  std::vector<image_t> image_ids;
-  image_ids.reserve(images_cache_.size());
-  for (const auto& image : images_cache_) {
-    image_ids.push_back(image.first);
-  }
-  return image_ids;
-}
-
-bool FeatureMatcherCache::ExistsKeypoints(const image_t image_id) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return keypoints_exists_cache_->Get(image_id);
-}
-
-bool FeatureMatcherCache::ExistsDescriptors(const image_t image_id) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return descriptors_exists_cache_->Get(image_id);
-}
-
-bool FeatureMatcherCache::ExistsMatches(const image_t image_id1,
-                                        const image_t image_id2) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return database_->ExistsMatches(image_id1, image_id2);
-}
-
-bool FeatureMatcherCache::ExistsInlierMatches(const image_t image_id1,
-                                              const image_t image_id2) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  return database_->ExistsInlierMatches(image_id1, image_id2);
-}
-
-void FeatureMatcherCache::WriteMatches(const image_t image_id1,
-                                       const image_t image_id2,
-                                       const FeatureMatches& matches) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  database_->WriteMatches(image_id1, image_id2, matches);
-}
-
-void FeatureMatcherCache::WriteTwoViewGeometry(
-    const image_t image_id1,
-    const image_t image_id2,
-    const TwoViewGeometry& two_view_geometry) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  database_->WriteTwoViewGeometry(image_id1, image_id2, two_view_geometry);
-}
-
-void FeatureMatcherCache::DeleteMatches(const image_t image_id1,
-                                        const image_t image_id2) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  database_->DeleteMatches(image_id1, image_id2);
-}
-
-void FeatureMatcherCache::DeleteInlierMatches(const image_t image_id1,
-                                              const image_t image_id2) {
-  std::lock_guard<std::mutex> lock(database_mutex_);
-  database_->DeleteInlierMatches(image_id1, image_id2);
-}
-
 FeatureMatcherWorker::FeatureMatcherWorker(
     const SiftMatchingOptions& matching_options,
     const TwoViewGeometryOptions& geometry_options,
@@ -179,7 +51,7 @@ FeatureMatcherWorker::FeatureMatcherWorker(
       cache_(cache),
       input_queue_(input_queue),
       output_queue_(output_queue) {
-  CHECK(matching_options_.Check());
+  THROW_CHECK(matching_options_.Check());
 
   prev_keypoints_image_ids_[0] = kInvalidImageId;
   prev_keypoints_image_ids_[1] = kInvalidImageId;
@@ -200,8 +72,8 @@ void FeatureMatcherWorker::SetMaxNumMatches(int max_num_matches) {
 void FeatureMatcherWorker::Run() {
   if (matching_options_.use_gpu) {
 #if !defined(COLMAP_CUDA_ENABLED)
-    CHECK(opengl_context_);
-    CHECK(opengl_context_->MakeCurrent());
+    THROW_CHECK_NOTNULL(opengl_context_);
+    THROW_CHECK(opengl_context_->MakeCurrent());
 #endif
   }
 
@@ -226,12 +98,12 @@ void FeatureMatcherWorker::Run() {
 
       if (!cache_->ExistsDescriptors(data.image_id1) ||
           !cache_->ExistsDescriptors(data.image_id2)) {
-        CHECK(output_queue_->Push(std::move(data)));
+        THROW_CHECK(output_queue_->Push(std::move(data)));
         continue;
       }
 
       if (matching_options_.guided_matching) {
-        matcher->MatchGuided(geometry_options_,
+        matcher->MatchGuided(geometry_options_.ransac_options.max_error,
                              GetKeypointsPtr(0, data.image_id1),
                              GetKeypointsPtr(1, data.image_id2),
                              GetDescriptorsPtr(0, data.image_id1),
@@ -243,15 +115,15 @@ void FeatureMatcherWorker::Run() {
                        &data.matches);
       }
 
-      CHECK(output_queue_->Push(std::move(data)));
+      THROW_CHECK(output_queue_->Push(std::move(data)));
     }
   }
 }
 
 std::shared_ptr<FeatureKeypoints> FeatureMatcherWorker::GetKeypointsPtr(
     const int index, const image_t image_id) {
-  CHECK_GE(index, 0);
-  CHECK_LE(index, 1);
+  THROW_CHECK_GE(index, 0);
+  THROW_CHECK_LE(index, 1);
   if (prev_keypoints_image_ids_[index] == image_id) {
     return nullptr;
   } else {
@@ -263,8 +135,8 @@ std::shared_ptr<FeatureKeypoints> FeatureMatcherWorker::GetKeypointsPtr(
 
 std::shared_ptr<FeatureDescriptors> FeatureMatcherWorker::GetDescriptorsPtr(
     const int index, const image_t image_id) {
-  CHECK_GE(index, 0);
-  CHECK_LE(index, 1);
+  THROW_CHECK_GE(index, 0);
+  THROW_CHECK_LE(index, 1);
   if (prev_descriptors_image_ids_[index] == image_id) {
     return nullptr;
   } else {
@@ -289,7 +161,7 @@ class VerifierWorker : public Thread {
         cache_(cache),
         input_queue_(input_queue),
         output_queue_(output_queue) {
-    CHECK(options_.Check());
+    THROW_CHECK(options_.Check());
   }
 
  protected:
@@ -305,7 +177,7 @@ class VerifierWorker : public Thread {
 
         if (data.matches.size() <
             static_cast<size_t>(options_.min_num_inliers)) {
-          CHECK(output_queue_->Push(std::move(data)));
+          THROW_CHECK(output_queue_->Push(std::move(data)));
           continue;
         }
 
@@ -323,7 +195,7 @@ class VerifierWorker : public Thread {
         data.two_view_geometry = EstimateTwoViewGeometry(
             camera1, points1, camera2, points2, data.matches, options_);
 
-        CHECK(output_queue_->Push(std::move(data)));
+        THROW_CHECK(output_queue_->Push(std::move(data)));
       }
     }
   }
@@ -347,20 +219,20 @@ FeatureMatcherController::FeatureMatcherController(
       database_(database),
       cache_(cache),
       is_setup_(false) {
-  CHECK(matching_options_.Check());
-  CHECK(geometry_options_.Check());
+  THROW_CHECK(matching_options_.Check());
+  THROW_CHECK(geometry_options_.Check());
 
   const int num_threads = GetEffectiveNumThreads(matching_options_.num_threads);
-  CHECK_GT(num_threads, 0);
+  THROW_CHECK_GT(num_threads, 0);
 
   std::vector<int> gpu_indices = CSVToVector<int>(matching_options_.gpu_index);
-  CHECK_GT(gpu_indices.size(), 0);
+  THROW_CHECK_GT(gpu_indices.size(), 0);
 
 #if defined(COLMAP_CUDA_ENABLED)
   if (matching_options_.use_gpu && gpu_indices.size() == 1 &&
       gpu_indices[0] == -1) {
     const int num_cuda_devices = GetNumCudaDevices();
-    CHECK_GT(num_cuda_devices, 0);
+    THROW_CHECK_GT(num_cuda_devices, 0);
     gpu_indices.resize(num_cuda_devices);
     std::iota(gpu_indices.begin(), gpu_indices.end(), 0);
   }
@@ -473,7 +345,8 @@ FeatureMatcherController::~FeatureMatcherController() {
 bool FeatureMatcherController::Setup() {
   // Minimize the amount of allocated GPU memory by computing the maximum number
   // of descriptors for any image over the whole database.
-  const int max_num_features = CHECK_NOTNULL(database_)->MaxNumKeypoints();
+  const int max_num_features =
+      THROW_CHECK_NOTNULL(database_)->MaxNumKeypoints();
   matching_options_.max_num_matches =
       std::min(matching_options_.max_num_matches, max_num_features);
 
@@ -510,9 +383,9 @@ bool FeatureMatcherController::Setup() {
 
 void FeatureMatcherController::Match(
     const std::vector<std::pair<image_t, image_t>>& image_pairs) {
-  CHECK_NOTNULL(database_);
-  CHECK_NOTNULL(cache_);
-  CHECK(is_setup_);
+  THROW_CHECK_NOTNULL(database_);
+  THROW_CHECK_NOTNULL(cache_);
+  THROW_CHECK(is_setup_);
 
   if (image_pairs.empty()) {
     return;
@@ -568,9 +441,9 @@ void FeatureMatcherController::Match(
     if (exists_matches) {
       data.matches = cache_->GetMatches(image_pair.first, image_pair.second);
       cache_->DeleteMatches(image_pair.first, image_pair.second);
-      CHECK(verifier_queue_.Push(std::move(data)));
+      THROW_CHECK(verifier_queue_.Push(std::move(data)));
     } else {
-      CHECK(matcher_queue_.Push(std::move(data)));
+      THROW_CHECK(matcher_queue_.Push(std::move(data)));
     }
   }
 
@@ -580,7 +453,7 @@ void FeatureMatcherController::Match(
 
   for (size_t i = 0; i < num_outputs; ++i) {
     auto output_job = output_queue_.Pop();
-    CHECK(output_job.IsValid());
+    THROW_CHECK(output_job.IsValid());
     auto& output = output_job.Data();
 
     if (output.matches.size() <
@@ -598,7 +471,7 @@ void FeatureMatcherController::Match(
         output.image_id1, output.image_id2, output.two_view_geometry);
   }
 
-  CHECK_EQ(output_queue_.Size(), 0);
+  THROW_CHECK_EQ(output_queue_.Size(), 0);
 }
 
 }  // namespace colmap
