@@ -57,7 +57,7 @@ ceres::LossFunction* BundleAdjustmentOptions::CreateLossFunction() const {
       loss_function = new ceres::CauchyLoss(loss_function_scale);
       break;
   }
-  CHECK_NOTNULL(loss_function);
+  THROW_CHECK_NOTNULL(loss_function);
   return loss_function;
 }
 
@@ -159,8 +159,8 @@ bool BundleAdjustmentConfig::HasConstantCamIntrinsics(
 }
 
 void BundleAdjustmentConfig::SetConstantCamPose(const image_t image_id) {
-  CHECK(HasImage(image_id));
-  CHECK(!HasConstantCamPositions(image_id));
+  THROW_CHECK(HasImage(image_id));
+  THROW_CHECK(!HasConstantCamPositions(image_id));
   constant_cam_poses_.insert(image_id);
 }
 
@@ -174,11 +174,11 @@ bool BundleAdjustmentConfig::HasConstantCamPose(const image_t image_id) const {
 
 void BundleAdjustmentConfig::SetConstantCamPositions(
     const image_t image_id, const std::vector<int>& idxs) {
-  CHECK_GT(idxs.size(), 0);
-  CHECK_LE(idxs.size(), 3);
-  CHECK(HasImage(image_id));
-  CHECK(!HasConstantCamPose(image_id));
-  CHECK(!VectorContainsDuplicateValues(idxs))
+  THROW_CHECK_GT(idxs.size(), 0);
+  THROW_CHECK_LE(idxs.size(), 3);
+  THROW_CHECK(HasImage(image_id));
+  THROW_CHECK(!HasConstantCamPose(image_id));
+  THROW_CHECK(!VectorContainsDuplicateValues(idxs))
       << "Tvec indices must not contain duplicates";
   constant_cam_positions_.emplace(image_id, idxs);
 }
@@ -194,6 +194,11 @@ bool BundleAdjustmentConfig::HasConstantCamPositions(
          constant_cam_positions_.end();
 }
 
+const std::unordered_set<camera_t> BundleAdjustmentConfig::ConstantIntrinsics()
+    const {
+  return constant_intrinsics_;
+}
+
 const std::unordered_set<image_t>& BundleAdjustmentConfig::Images() const {
   return image_ids_;
 }
@@ -208,18 +213,23 @@ const std::unordered_set<point3D_t>& BundleAdjustmentConfig::ConstantPoints()
   return constant_point3D_ids_;
 }
 
+const std::unordered_set<image_t>& BundleAdjustmentConfig::ConstantCamPoses()
+    const {
+  return constant_cam_poses_;
+}
+
 const std::vector<int>& BundleAdjustmentConfig::ConstantCamPositions(
     const image_t image_id) const {
   return constant_cam_positions_.at(image_id);
 }
 
 void BundleAdjustmentConfig::AddVariablePoint(const point3D_t point3D_id) {
-  CHECK(!HasConstantPoint(point3D_id));
+  THROW_CHECK(!HasConstantPoint(point3D_id));
   variable_point3D_ids_.insert(point3D_id);
 }
 
 void BundleAdjustmentConfig::AddConstantPoint(const point3D_t point3D_id) {
-  CHECK(!HasVariablePoint(point3D_id));
+  THROW_CHECK(!HasVariablePoint(point3D_id));
   constant_point3D_ids_.insert(point3D_id);
 }
 
@@ -252,78 +262,52 @@ void BundleAdjustmentConfig::RemoveConstantPoint(const point3D_t point3D_id) {
 BundleAdjuster::BundleAdjuster(const BundleAdjustmentOptions& options,
                                const BundleAdjustmentConfig& config)
     : options_(options), config_(config) {
-  CHECK(options_.Check());
+  THROW_CHECK(options_.Check());
 }
 
 bool BundleAdjuster::Solve(Reconstruction* reconstruction) {
-  CHECK_NOTNULL(reconstruction);
-  CHECK(!problem_) << "Cannot use the same BundleAdjuster multiple times";
-
-  ceres::Problem::Options problem_options;
-  problem_options.loss_function_ownership = ceres::DO_NOT_TAKE_OWNERSHIP;
-  problem_ = std::make_unique<ceres::Problem>(problem_options);
-
-  const auto loss_function =
+  loss_function_ =
       std::unique_ptr<ceres::LossFunction>(options_.CreateLossFunction());
-  SetUp(reconstruction, loss_function.get());
+  SetUpProblem(reconstruction, loss_function_.get());
 
   if (problem_->NumResiduals() == 0) {
     return false;
   }
 
-  ceres::Solver::Options solver_options = options_.solver_options;
-  const bool has_sparse =
-      solver_options.sparse_linear_algebra_library_type != ceres::NO_SPARSE;
-
-  // Empirical choice.
-  const size_t kMaxNumImagesDirectDenseSolver = 50;
-  const size_t kMaxNumImagesDirectSparseSolver = 1000;
-  const size_t num_images = config_.NumImages();
-  if (num_images <= kMaxNumImagesDirectDenseSolver) {
-    solver_options.linear_solver_type = ceres::DENSE_SCHUR;
-  } else if (num_images <= kMaxNumImagesDirectSparseSolver && has_sparse) {
-    solver_options.linear_solver_type = ceres::SPARSE_SCHUR;
-  } else {  // Indirect sparse (preconditioned CG) solver.
-    solver_options.linear_solver_type = ceres::ITERATIVE_SCHUR;
-    solver_options.preconditioner_type = ceres::SCHUR_JACOBI;
-  }
-
-  if (problem_->NumResiduals() <
-      options_.min_num_residuals_for_multi_threading) {
-    solver_options.num_threads = 1;
-#if CERES_VERSION_MAJOR < 2
-    solver_options.num_linear_solver_threads = 1;
-#endif  // CERES_VERSION_MAJOR
-  } else {
-    solver_options.num_threads =
-        GetEffectiveNumThreads(solver_options.num_threads);
-#if CERES_VERSION_MAJOR < 2
-    solver_options.num_linear_solver_threads =
-        GetEffectiveNumThreads(solver_options.num_linear_solver_threads);
-#endif  // CERES_VERSION_MAJOR
-  }
-
-  std::string solver_error;
-  CHECK(solver_options.IsValid(&solver_error)) << solver_error;
+  ceres::Solver::Options solver_options =
+      SetUpSolverOptions(*problem_, options_.solver_options);
 
   ceres::Solve(solver_options, problem_.get(), &summary_);
 
-  if (options_.print_summary) {
-    PrintHeading2("Bundle adjustment report");
-    PrintSolverSummary(summary_);
+  if (options_.print_summary || VLOG_IS_ON(1)) {
+    PrintSolverSummary(summary_, "Bundle adjustment report");
   }
 
-  TearDown(reconstruction);
-
   return true;
 }
 
+const BundleAdjustmentOptions& BundleAdjuster::Options() const {
+  return options_;
+}
+
+const BundleAdjustmentConfig& BundleAdjuster::Config() const { return config_; }
+
+std::shared_ptr<ceres::Problem> BundleAdjuster::Problem() { return problem_; }
+
 const ceres::Solver::Summary& BundleAdjuster::Summary() const {
   return summary_;
 }
 
-void BundleAdjuster::SetUp(Reconstruction* reconstruction,
-                           ceres::LossFunction* loss_function) {
+void BundleAdjuster::SetUpProblem(Reconstruction* reconstruction,
+                                  ceres::LossFunction* loss_function) {
+  THROW_CHECK_NOTNULL(reconstruction);
+
+  // Initialize an empty problem
+  ceres::Problem::Options problem_options;
+  problem_options.loss_function_ownership = ceres::DO_NOT_TAKE_OWNERSHIP;
+  problem_ = std::make_shared<ceres::Problem>(problem_options);
+
+  // Set up problem
   // Warning: AddPointsToProblem assumes that AddImageToProblem is called first.
   // Do not change order of instructions!
   for (const image_t image_id : config_.Images()) {
@@ -340,8 +324,43 @@ void BundleAdjuster::SetUp(Reconstruction* reconstruction,
   ParameterizePoints(reconstruction);
 }
 
-void BundleAdjuster::TearDown(Reconstruction*) {
-  // Nothing to do
+ceres::Solver::Options BundleAdjuster::SetUpSolverOptions(
+    const ceres::Problem& problem,
+    const ceres::Solver::Options& input_solver_options) const {
+  ceres::Solver::Options solver_options = input_solver_options;
+  const bool has_sparse =
+      solver_options.sparse_linear_algebra_library_type != ceres::NO_SPARSE;
+
+  // Empirical choice.
+  const size_t kMaxNumImagesDirectDenseSolver = 50;
+  const size_t kMaxNumImagesDirectSparseSolver = 1000;
+  const size_t num_images = config_.NumImages();
+  if (num_images <= kMaxNumImagesDirectDenseSolver) {
+    solver_options.linear_solver_type = ceres::DENSE_SCHUR;
+  } else if (num_images <= kMaxNumImagesDirectSparseSolver && has_sparse) {
+    solver_options.linear_solver_type = ceres::SPARSE_SCHUR;
+  } else {  // Indirect sparse (preconditioned CG) solver.
+    solver_options.linear_solver_type = ceres::ITERATIVE_SCHUR;
+    solver_options.preconditioner_type = ceres::SCHUR_JACOBI;
+  }
+
+  if (problem.NumResiduals() < options_.min_num_residuals_for_multi_threading) {
+    solver_options.num_threads = 1;
+#if CERES_VERSION_MAJOR < 2
+    solver_options.num_linear_solver_threads = 1;
+#endif  // CERES_VERSION_MAJOR
+  } else {
+    solver_options.num_threads =
+        GetEffectiveNumThreads(solver_options.num_threads);
+#if CERES_VERSION_MAJOR < 2
+    solver_options.num_linear_solver_threads =
+        GetEffectiveNumThreads(solver_options.num_linear_solver_threads);
+#endif  // CERES_VERSION_MAJOR
+  }
+
+  std::string solver_error;
+  THROW_CHECK(solver_options.IsValid(&solver_error)) << solver_error;
+  return solver_options;
 }
 
 void BundleAdjuster::AddImageToProblem(const image_t image_id,
@@ -374,36 +393,16 @@ void BundleAdjuster::AddImageToProblem(const image_t image_id,
     Point3D& point3D = reconstruction->Point3D(point2D.point3D_id);
     assert(point3D.track.Length() > 1);
 
-    ceres::CostFunction* cost_function = nullptr;
-
     if (constant_cam_pose) {
-      switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                                        \
-  case CameraModel::model_id:                                                 \
-    cost_function = ReprojErrorConstantPoseCostFunction<CameraModel>::Create( \
-        image.CamFromWorld(), point2D.xy);                                    \
-    break;
-
-        CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-      }
-
       problem_->AddResidualBlock(
-          cost_function, loss_function, point3D.xyz.data(), camera_params);
+          CameraCostFunction<ReprojErrorConstantPoseCostFunction>(
+              camera.model_id, image.CamFromWorld(), point2D.xy),
+          loss_function,
+          point3D.xyz.data(),
+          camera_params);
     } else {
-      switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                                        \
-  case CameraModel::model_id:                                                 \
-    cost_function = ReprojErrorCostFunction<CameraModel>::Create(point2D.xy); \
-    break;
-
-        CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-      }
-
-      problem_->AddResidualBlock(cost_function,
+      problem_->AddResidualBlock(CameraCostFunction<ReprojErrorCostFunction>(
+                                     camera.model_id, point2D.xy),
                                  loss_function,
                                  cam_from_world_rotation,
                                  cam_from_world_translation,
@@ -464,22 +463,12 @@ void BundleAdjuster::AddPointToProblem(const point3D_t point3D_id,
       camera_ids_.insert(image.CameraId());
       config_.SetConstantCamIntrinsics(image.CameraId());
     }
-
-    ceres::CostFunction* cost_function = nullptr;
-
-    switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                                        \
-  case CameraModel::model_id:                                                 \
-    cost_function = ReprojErrorConstantPoseCostFunction<CameraModel>::Create( \
-        image.CamFromWorld(), point2D.xy);                                    \
-    break;
-
-      CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-    }
     problem_->AddResidualBlock(
-        cost_function, loss_function, point3D.xyz.data(), camera.params.data());
+        CameraCostFunction<ReprojErrorConstantPoseCostFunction>(
+            camera.model_id, image.CamFromWorld(), point2D.xy),
+        loss_function,
+        point3D.xyz.data(),
+        camera.params.data());
   }
 }
 
@@ -547,75 +536,21 @@ RigBundleAdjuster::RigBundleAdjuster(const BundleAdjustmentOptions& options,
 
 bool RigBundleAdjuster::Solve(Reconstruction* reconstruction,
                               std::vector<CameraRig>* camera_rigs) {
-  CHECK_NOTNULL(reconstruction);
-  CHECK_NOTNULL(camera_rigs);
-  CHECK(!problem_) << "Cannot use the same BundleAdjuster multiple times";
-
-  // Check the validity of the provided camera rigs.
-  std::unordered_set<camera_t> rig_camera_ids;
-  for (auto& camera_rig : *camera_rigs) {
-    camera_rig.Check(*reconstruction);
-    for (const auto& camera_id : camera_rig.GetCameraIds()) {
-      CHECK_EQ(rig_camera_ids.count(camera_id), 0)
-          << "Camera must not be part of multiple camera rigs";
-      rig_camera_ids.insert(camera_id);
-    }
-
-    for (const auto& snapshot : camera_rig.Snapshots()) {
-      for (const auto& image_id : snapshot) {
-        CHECK_EQ(image_id_to_camera_rig_.count(image_id), 0)
-            << "Image must not be part of multiple camera rigs";
-        image_id_to_camera_rig_.emplace(image_id, &camera_rig);
-      }
-    }
-  }
-
-  problem_ = std::make_unique<ceres::Problem>();
-
-  ceres::Problem::Options problem_options;
-  problem_options.loss_function_ownership = ceres::DO_NOT_TAKE_OWNERSHIP;
-  problem_ = std::make_unique<ceres::Problem>(problem_options);
-
-  const auto loss_function =
+  loss_function_ =
       std::unique_ptr<ceres::LossFunction>(options_.CreateLossFunction());
-  SetUp(reconstruction, camera_rigs, loss_function.get());
+  SetUpProblem(reconstruction, camera_rigs, loss_function_.get());
 
   if (problem_->NumResiduals() == 0) {
     return false;
   }
 
-  ceres::Solver::Options solver_options = options_.solver_options;
-  const bool has_sparse =
-      solver_options.sparse_linear_algebra_library_type != ceres::NO_SPARSE;
-
-  // Empirical choice.
-  const size_t kMaxNumImagesDirectDenseSolver = 50;
-  const size_t kMaxNumImagesDirectSparseSolver = 1000;
-  const size_t num_images = config_.NumImages();
-  if (num_images <= kMaxNumImagesDirectDenseSolver) {
-    solver_options.linear_solver_type = ceres::DENSE_SCHUR;
-  } else if (num_images <= kMaxNumImagesDirectSparseSolver && has_sparse) {
-    solver_options.linear_solver_type = ceres::SPARSE_SCHUR;
-  } else {  // Indirect sparse (preconditioned CG) solver.
-    solver_options.linear_solver_type = ceres::ITERATIVE_SCHUR;
-    solver_options.preconditioner_type = ceres::SCHUR_JACOBI;
-  }
-
-  solver_options.num_threads =
-      GetEffectiveNumThreads(solver_options.num_threads);
-#if CERES_VERSION_MAJOR < 2
-  solver_options.num_linear_solver_threads =
-      GetEffectiveNumThreads(solver_options.num_linear_solver_threads);
-#endif  // CERES_VERSION_MAJOR
-
-  std::string solver_error;
-  CHECK(solver_options.IsValid(&solver_error)) << solver_error;
+  ceres::Solver::Options solver_options =
+      SetUpSolverOptions(*problem_, options_.solver_options);
 
   ceres::Solve(solver_options, problem_.get(), &summary_);
 
-  if (options_.print_summary) {
-    PrintHeading2("Rig Bundle adjustment report");
-    PrintSolverSummary(summary_);
+  if (options_.print_summary || VLOG_IS_ON(1)) {
+    PrintSolverSummary(summary_, "Rig Bundle adjustment report");
   }
 
   TearDown(reconstruction, *camera_rigs);
@@ -623,9 +558,39 @@ bool RigBundleAdjuster::Solve(Reconstruction* reconstruction,
   return true;
 }
 
-void RigBundleAdjuster::SetUp(Reconstruction* reconstruction,
-                              std::vector<CameraRig>* camera_rigs,
-                              ceres::LossFunction* loss_function) {
+void RigBundleAdjuster::SetUpProblem(Reconstruction* reconstruction,
+                                     std::vector<CameraRig>* camera_rigs,
+                                     ceres::LossFunction* loss_function) {
+  THROW_CHECK_NOTNULL(reconstruction);
+  THROW_CHECK_NOTNULL(camera_rigs);
+  THROW_CHECK(!problem_)
+      << "Cannot set up problem from the same BundleAdjuster multiple times";
+
+  // Check the validity of the provided camera rigs.
+  std::unordered_set<camera_t> rig_camera_ids;
+  for (auto& camera_rig : *camera_rigs) {
+    camera_rig.Check(*reconstruction);
+    for (const auto& camera_id : camera_rig.GetCameraIds()) {
+      THROW_CHECK_EQ(rig_camera_ids.count(camera_id), 0)
+          << "Camera must not be part of multiple camera rigs";
+      rig_camera_ids.insert(camera_id);
+    }
+
+    for (const auto& snapshot : camera_rig.Snapshots()) {
+      for (const auto& image_id : snapshot) {
+        THROW_CHECK_EQ(image_id_to_camera_rig_.count(image_id), 0)
+            << "Image must not be part of multiple camera rigs";
+        image_id_to_camera_rig_.emplace(image_id, &camera_rig);
+      }
+    }
+  }
+
+  // Initialize an empty problem
+  ceres::Problem::Options problem_options;
+  problem_options.loss_function_ownership = ceres::DO_NOT_TAKE_OWNERSHIP;
+  problem_ = std::make_shared<ceres::Problem>(problem_options);
+
+  // Set up problem
   ComputeCameraRigPoses(*reconstruction, *camera_rigs);
 
   for (const image_t image_id : config_.Images()) {
@@ -676,9 +641,9 @@ void RigBundleAdjuster::AddImageToProblem(const image_t image_id,
   Eigen::Matrix3x4d cam_from_world_mat = Eigen::Matrix3x4d::Zero();
 
   if (image_id_to_camera_rig_.count(image_id) > 0) {
-    CHECK(!constant_cam_pose)
+    THROW_CHECK(!constant_cam_pose)
         << "Images contained in a camera rig must not have constant pose";
-    CHECK(!constant_cam_position)
+    THROW_CHECK(!constant_cam_position)
         << "Images contained in a camera rig must not have constant tvec";
     camera_rig = image_id_to_camera_rig_.at(image_id);
     Rigid3d& rig_from_world = *image_id_to_rig_from_world_.at(image_id);
@@ -696,7 +661,7 @@ void RigBundleAdjuster::AddImageToProblem(const image_t image_id,
   }
 
   // Collect cameras for final parameterization.
-  CHECK(image.HasCamera());
+  THROW_CHECK(image.HasCamera());
   camera_ids_.insert(image.CameraId());
 
   // The number of added observations for the current image.
@@ -721,37 +686,17 @@ void RigBundleAdjuster::AddImageToProblem(const image_t image_id,
     num_observations += 1;
     point3D_num_observations_[point2D.point3D_id] += 1;
 
-    ceres::CostFunction* cost_function = nullptr;
-
     if (camera_rig == nullptr) {
       if (constant_cam_pose) {
-        switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                                        \
-  case CameraModel::model_id:                                                 \
-    cost_function = ReprojErrorConstantPoseCostFunction<CameraModel>::Create( \
-        image.CamFromWorld(), point2D.xy);                                    \
-    break;
-
-          CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-        }
-
         problem_->AddResidualBlock(
-            cost_function, loss_function, point3D.xyz.data(), camera_params);
+            CameraCostFunction<ReprojErrorConstantPoseCostFunction>(
+                camera.model_id, image.CamFromWorld(), point2D.xy),
+            loss_function,
+            point3D.xyz.data(),
+            camera_params);
       } else {
-        switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                                        \
-  case CameraModel::model_id:                                                 \
-    cost_function = ReprojErrorCostFunction<CameraModel>::Create(point2D.xy); \
-    break;
-
-          CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-        }
-
-        problem_->AddResidualBlock(cost_function,
+        problem_->AddResidualBlock(CameraCostFunction<ReprojErrorCostFunction>(
+                                       camera.model_id, point2D.xy),
                                    loss_function,
                                    cam_from_rig_rotation,     // rig == world
                                    cam_from_rig_translation,  // rig == world
@@ -759,19 +704,8 @@ void RigBundleAdjuster::AddImageToProblem(const image_t image_id,
                                    camera_params);
       }
     } else {
-      switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                               \
-  case CameraModel::model_id:                                        \
-    cost_function =                                                  \
-        RigReprojErrorCostFunction<CameraModel>::Create(point2D.xy); \
-                                                                     \
-    break;
-
-        CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-      }
-      problem_->AddResidualBlock(cost_function,
+      problem_->AddResidualBlock(CameraCostFunction<RigReprojErrorCostFunction>(
+                                     camera.model_id, point2D.xy),
                                  loss_function,
                                  cam_from_rig_rotation,
                                  cam_from_rig_translation,
@@ -840,23 +774,12 @@ void RigBundleAdjuster::AddPointToProblem(const point3D_t point3D_id,
       config_.SetConstantCamIntrinsics(image.CameraId());
     }
 
-    ceres::CostFunction* cost_function = nullptr;
-
-    switch (camera.model_id) {
-#define CAMERA_MODEL_CASE(CameraModel)                                        \
-  case CameraModel::model_id:                                                 \
-    cost_function = ReprojErrorConstantPoseCostFunction<CameraModel>::Create( \
-        image.CamFromWorld(), point2D.xy);                                    \
-    problem_->AddResidualBlock(cost_function,                                 \
-                               loss_function,                                 \
-                               point3D.xyz.data(),                            \
-                               camera.params.data());                         \
-    break;
-
-      CAMERA_MODEL_SWITCH_CASES
-
-#undef CAMERA_MODEL_CASE
-    }
+    problem_->AddResidualBlock(
+        CameraCostFunction<ReprojErrorConstantPoseCostFunction>(
+            camera.model_id, image.CamFromWorld(), point2D.xy),
+        loss_function,
+        point3D.xyz.data(),
+        camera.params.data());
   }
 }
 
@@ -887,9 +810,10 @@ void RigBundleAdjuster::ParameterizeCameraRigs(Reconstruction* reconstruction) {
   }
 }
 
-void PrintSolverSummary(const ceres::Solver::Summary& summary) {
+void PrintSolverSummary(const ceres::Solver::Summary& summary,
+                        const std::string& header) {
   std::ostringstream log;
-  log << "\n";
+  log << "\n" << header << ":\n";
   log << std::right << std::setw(16) << "Residuals : ";
   log << std::left << summary.num_residuals_reduced << "\n";
 
