@@ -502,19 +502,12 @@ ImageTab::ImageTab(QWidget* parent,
   grid->addWidget(overlapping_images_button, 0, 4, Qt::AlignRight);
 
   table_widget_ = new QTableWidget(this);
-  table_widget_->setColumnCount(10);
+  table_widget_->setColumnCount(3);
 
   QStringList table_header;
   table_header << "image_id"
                << "name"
-               << "camera_id"
-               << "qw"
-               << "qx"
-               << "qy"
-               << "qz"
-               << "tx"
-               << "ty"
-               << "tz";
+               << "camera_id";
   table_widget_->setHorizontalHeaderLabels(table_header);
 
   table_widget_->setShowGrid(true);
@@ -560,34 +553,6 @@ void ImageTab::Reload() {
         i, 1, new QTableWidgetItem(QString::fromStdString(image.Name())));
     table_widget_->setItem(
         i, 2, new QTableWidgetItem(QString::number(image.CameraId())));
-    table_widget_->setItem(i,
-                           3,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().rotation.w())));
-    table_widget_->setItem(i,
-                           4,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().rotation.x())));
-    table_widget_->setItem(i,
-                           5,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().rotation.y())));
-    table_widget_->setItem(i,
-                           6,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().rotation.z())));
-    table_widget_->setItem(i,
-                           7,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().translation.x())));
-    table_widget_->setItem(i,
-                           8,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().translation.y())));
-    table_widget_->setItem(i,
-                           9,
-                           new QTableWidgetItem(QString::number(
-                               image.CamFromWorldPrior().translation.z())));
   }
   table_widget_->resizeColumnsToContents();
 
@@ -619,34 +584,6 @@ void ImageTab::itemChanged(QTableWidgetItem* item) {
         image.SetCameraId(camera_id);
       }
       break;
-    case 3:
-      image.CamFromWorldPrior().rotation.w() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
-    case 4:
-      image.CamFromWorldPrior().rotation.x() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
-    case 5:
-      image.CamFromWorldPrior().rotation.y() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
-    case 6:
-      image.CamFromWorldPrior().rotation.z() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
-    case 7:
-      image.CamFromWorldPrior().translation.x() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
-    case 8:
-      image.CamFromWorldPrior().translation.y() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
-    case 9:
-      image.CamFromWorldPrior().translation.z() =
-          item->data(Qt::DisplayRole).toReal();
-      break;
     default:
       break;
   }
